﻿using System;
using System.Data.Common;
using EPiServer.Common.Data;
using EPiServer.Common.Security;

namespace ExampleProjects.Movie.Data
{
    public class MovieFactory : FrameworkFactoryBase
    {
        internal static Movie GetMovie(int id)
        {
            Movie movie = null;
            
            using (DbDataReader reader = 
                DatabaseHandler.GetReader("spGetMovie", id))
            {
                if (reader.Read())
                    movie = ConstructMovie(reader);
            }

            return movie;
        }

        internal static Movie AddMovie(Movie movie)
        {
            bool wasInTransaction = DatabaseHandler.InTransaction;
            if (!wasInTransaction)
                DatabaseHandler.BeginTransaction();

            int newId = -1;
            try
            {
                object[] spParameters = new object[3];
                spParameters[0] = movie.Title;
                spParameters[1] = movie.RuntimeMinutes;
                spParameters[2] = movie.Author.ID;

                newId = Convert.ToInt32(DatabaseHandler.GetScalar(
                    "spAddMovie", spParameters));
                
                UpdateEntity(movie, newId);

                Movie addedMovie = MovieHandler.GetMovie(newId);
                MovieHandler.OnMovieAdded(addedMovie);

                if (!wasInTransaction)
                    DatabaseHandler.Commit();
            }
            catch (Exception)
            {
                if (!wasInTransaction)
                    DatabaseHandler.Rollback();
                throw;
            }

            return MovieHandler.GetMovie(newId);
        }

        internal static void UpdateMovie(Movie movie)
        {
            bool wasInTransaction = DatabaseHandler.InTransaction;
            if (!wasInTransaction)
                DatabaseHandler.BeginTransaction();

            try
            {
                object[] spParameters = new object[4];
                spParameters[0] = movie.ID;
                spParameters[1] = movie.Title;
                spParameters[2] = movie.RuntimeMinutes;
                if (movie.Author != null)
                    spParameters[3] = movie.Author.ID;
                else
                    spParameters[3] = -1;

                DatabaseHandler.ExecuteNonQuery("spUpdateMovie", spParameters);
                
                UpdateEntity(movie);
                
                MovieHandler.OnMovieUpdated(movie);

                if (!wasInTransaction)
                    DatabaseHandler.Commit();
            }
            catch (Exception)
            {
                if (!wasInTransaction)
                    DatabaseHandler.Rollback();
                throw;
            }
        }

        internal static void RemoveMovie(Movie movie)
        {
            bool wasInTransaction = DatabaseHandler.InTransaction;
            if (!wasInTransaction)
                DatabaseHandler.BeginTransaction();

            try
            {
                MovieHandler.OnMovieRemoved(movie);

                DatabaseHandler.ExecuteNonQuery("spRemoveMovie", movie.ID);
                
                RemoveEntity(movie);
                if (!wasInTransaction)
                    DatabaseHandler.Commit();
            }
            catch (Exception)
            {
                if (!wasInTransaction)
                    DatabaseHandler.Rollback();
                throw;
            }
        }

        internal static PagedCollectionResultSet GetMoviesByAuthor(
            IUser author, int page, int pageSize)
        {
            object[] spParameters = new object[3];
            spParameters[0] = author.ID;
            spParameters[1] = page;
            spParameters[2] = pageSize;

            int totalItems = 0;
            MovieCollection movies = new MovieCollection();
            using (DbDataReader reader = 
                DatabaseHandler.GetReader(
                "spGetMoviesByAuthor", spParameters))
            {
                while (reader.Read())
                {
                    if(totalItems == 0)
                    {
                        int ordinal = 
                            reader.GetOrdinal("intTotalItems");
                        totalItems = reader.GetInt32(ordinal);
                    }
                    movies.Add(ConstructMovie(reader));
                }
            }

            PagedCollectionResultSet pagedMovies = 
                new PagedCollectionResultSet(movies, totalItems);

            return pagedMovies;
        }

        private static Movie ConstructMovie(DbDataReader reader)
        {
            IEntityProvider movieEntityProvider = 
                EntityProviderHandler.GetEntityProvider(typeof (Movie));

            return (Movie)
                movieEntityProvider.GetEntityInstance(
                    typeof(Movie), reader);
        }
    }
}
